@@ -1659,11 +1659,11 @@ gnome_canvas_rich_text_set_attributes_from_style(GnomeCanvasRichText *text,
 	values->appearance.bg_color = style->base[GTK_STATE_NORMAL];
 	values->appearance.fg_color = style->fg[GTK_STATE_NORMAL];
 	
-	if (values->font.family_name)
-		g_free (values->font.family_name);
-	
-	values->font = *style->font_desc;
-	values->font.family_name = g_strdup (style->font_desc->family_name);
+	if (values->font)
+		pango_font_description_free (values->font);
+
+	values->font = pango_font_description_copy (style->font_desc);
+
 } /* gnome_canvas_rich_text_set_attributes_from_style */
 
 static void
@@ -1930,6 +1930,7 @@ gnome_canvas_rich_text_draw(GnomeCanvasItem *item, GdkDrawable *drawable,
 		text->_priv->layout,
 		GTK_WIDGET(item->canvas),
 		drawable,
+		GTK_WIDGET (item->canvas)->style->text_gc[GTK_STATE_NORMAL],
 		x - x1, y - y1,
 		0, 0, (x2 - x1) - (x - x1), (y2 - y1) - (y - y1));
 } /* gnome_canvas_rich_text_draw */
@@ -1940,6 +1941,7 @@ gnome_canvas_rich_text_render(GnomeCanvasItem *item, GnomeCanvasBuf *buf)
 	g_warning ("rich text item not implemented for anti-aliased canvas");
 } /* gnome_canvas_rich_text_render */
 
+#if 0
 static GtkTextTag *
 gnome_canvas_rich_text_add_tag(GnomeCanvasRichText *text, char *tag_name,
 			       int start_offset, int end_offset, 
@@ -1975,3 +1977,4 @@ gnome_canvas_rich_text_add_tag(GnomeCanvasRichText *text, char *tag_name,
 
 	return tag;
 } /* gnome_canvas_rich_text_add_tag */
+#endif
